/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/ansi/string.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( SCO_STRING_CHECK )
#ifndef __GNUC__
extern "C++" {
inline void *memchr(void *__1, int __2, size_t __3)
	{ return (void *)memchr((const void *)__1, __2, __3); }
inline char *strstr(char *__1, const char *__2)
	{ return (char *)strstr((const char *)__1, __2); }
}
#endif /* ! __GNUC__ */

#endif  /* SCO_STRING_CHECK */
