/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/X11/Intrinsic.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( ARM_NORCROFT_HINT_CHECK )
p_type mumble;
#endif  /* ARM_NORCROFT_HINT_CHECK */
